@@ -53,7 +53,7 @@ QT_BEGIN_NAMESPACE
 /*!
     \qmltype SwipeDelegate
     \inherits ItemDelegate
-    \instantiates QQuickSwipeDelegate
+//!     \instantiates QQuickSwipeDelegate
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \ingroup qtquickcontrols2-delegates
@@ -697,6 +697,11 @@ void QQuickSwipe::close()
     if (qFuzzyIsNull(d->position))
         return;
 
+    if (d->control->isPressed()) {
+        // We don't support closing when we're pressed; release() or clicked() should be used instead.
+        return;
+    }
+
     d->beginTransition(0.0);
     d->wasComplete = false;
     d->positionBeforePress = 0.0;
@@ -708,6 +713,29 @@ QQuickSwipeDelegatePrivate::QQuickSwipeDelegatePrivate(QQuickSwipeDelegate *cont
 {
 }
 
+void QQuickSwipeDelegatePrivate::resizeBackground()
+{
+    if (!background)
+        return;
+
+    resizingBackground = true;
+
+    QQuickItemPrivate *p = QQuickItemPrivate::get(background);
+    const bool extraAllocated = extra.isAllocated();
+    // Don't check for or set the x here since it will just be overwritten by reposition().
+    if (((!p->widthValid || !extraAllocated || !extra->hasBackgroundWidth))
+            || (extraAllocated && (extra->hasLeftInset || extra->hasRightInset))) {
+        background->setWidth(width - getLeftInset() - getRightInset());
+    }
+    if (((!p->heightValid || !extraAllocated || !extra->hasBackgroundHeight) && qFuzzyIsNull(background->y()))
+            || (extraAllocated && (extra->hasTopInset || extra->hasBottomInset))) {
+        background->setY(getTopInset());
+        background->setHeight(height - getTopInset() - getBottomInset());
+    }
+
+    resizingBackground = false;
+}
+
 bool QQuickSwipeDelegatePrivate::handleMousePressEvent(QQuickItem *item, QMouseEvent *event)
 {
     Q_Q(QQuickSwipeDelegate);
@@ -889,7 +917,11 @@ bool QQuickSwipeDelegatePrivate::handleMouseReleaseEvent(QQuickItem *item, QMous
         swipePrivate->beginTransition(-1.0);
         swipePrivate->wasComplete = true;
     } else if (!swipePrivate->isTransitioning()) {
-        swipePrivate->beginTransition(0.0);
+        // The position is either <= 0.5 or >= -0.5, so the position should go to 0.
+        // However, if the position was already 0 or close to it, we were just clicked,
+        // and we don't need to start a transition.
+        if (!qFuzzyIsNull(swipePrivate->position))
+            swipePrivate->beginTransition(0.0);
         swipePrivate->wasComplete = false;
     }
 
@@ -926,13 +958,15 @@ void QQuickSwipeDelegatePrivate::resizeContent()
     // If the background and contentItem are repositioned due to a swipe,
     // we don't want to call QQuickControlPrivate's implementation of this function,
     // as it repositions the contentItem to be visible.
-    // However, we still want to resize the control vertically.
+    // However, we still want to position the contentItem vertically
+    // and resize it (in case the control was resized while open).
     QQuickSwipePrivate *swipePrivate = QQuickSwipePrivate::get(&swipe);
     if (!swipePrivate->complete) {
         QQuickItemDelegatePrivate::resizeContent();
     } else if (contentItem) {
         Q_Q(QQuickSwipeDelegate);
         contentItem->setY(q->topPadding());
+        contentItem->setWidth(q->availableWidth());
         contentItem->setHeight(q->availableHeight());
     }
 }
@@ -1016,7 +1050,6 @@ QQuickSwipeDelegate::QQuickSwipeDelegate(QQuickItem *parent)
 */
 
 /*!
-    \qmlpropertygroup QtQuick.Controls::SwipeDelegate::swipe
     \qmlproperty real QtQuick.Controls::SwipeDelegate::swipe.position
     \qmlproperty bool QtQuick.Controls::SwipeDelegate::swipe.complete
     \qmlproperty bool QtQuick.Controls::SwipeDelegate::swipe.enabled
